﻿package  
{
	import com.pblabs.box2D.CollisionEvent;
	import com.pblabs.engine.PBE;
	import com.pblabs.rendering2D.SpriteRenderer;
	import com.pblabs.engine.entity.IEntity;
	import com.pblabs.rendering2D.SpriteSheetRenderer;
	import com.pblabs.components.basic.HealthComponent;
	import com.pblabs.engine.components.TickedComponent;
	import com.pblabs.engine.entity.PropertyReference;
	import com.pblabs.engine.core.InputKey;
	import com.pblabs.engine.components.ThinkingComponent;
	
	import flash.events.Event;
	import flash.geom.Point;
	import flash.media.Sound;
	
	import pawnScreens.PawnGame;
	
	/**
	 * ...
	 * @author Jesse James Lord
	 */
	
	public class PawnKeyboardController extends TickedComponent
	{
		public var RenderReference:PropertyReference = null;   
		public var VelocityReference:PropertyReference = null;
		public var PositionReference:PropertyReference = null;
		public var AnimationEventName:String;
		public var Speed:Number = 0;
		public var JumpSpeed:Number = 0;
		public var WeaponDelay:Number = 0;
		public var WeaponOffset:Point = null;
		
		protected var facing:Boolean = true;
		protected var idle:Boolean = true;
		protected var hurtSound:Sound = null;
		protected var isJumping:Boolean = false;
		protected var onGround:int = 1;
		protected var playerIsDead:Boolean = false;
		protected var playerChangeLevel:Boolean = false;
		
		static public var weaponDelay:Number = 0;
		static public var inMenu:Boolean = false;
		static public var inAttack:Boolean = false;
		static public var playerPosition:Point = null;
		static public var animation:String;
		
		public function PawnKeyboardController():void
		{
			PawnGame.lblHealth.alpha = 1;
			inMenu = false;
		}
		
		public function get currentAnimation():String
		{
			return animation;
		}
		
		public override function onTick(tickRate:Number):void
		{
			//If we are dead then wait for level cleanup or retry
			if ( playerIsDead || playerChangeLevel )
			{
				return;
			}
			else
			{
				var nameString:String = new String;
				nameString = "pawnHero" + PBE.levelManager.currentLevel;
				//Update mainPlayer's health
				//Get the player entity
				var pawnHealth:HealthComponent = PBE.lookupComponentByName(nameString, "pawnPlayerHealth" ) as HealthComponent;
				if ( pawnHealth.health < pawnHealth.maxHealth)
				{
					//lets give our guy a little regen here
					pawnHealth.health = pawnHealth.health + 0.01;
				}
				//Update the color of the health
				if ( pawnHealth.health > 75 )
					PawnGame.lblHealth.fontColor = 0x00FF00;//green
				else if ( pawnHealth.health > 45 )
					PawnGame.lblHealth.fontColor = 0xFFFF00;//yellow
				else
					PawnGame.lblHealth.fontColor = 0xFF0000;//red
				//put the new health total on screen	
				PawnGame.lblHealth.caption = "Health: " + Math.round(pawnHealth.health) + " / " + pawnHealth.maxHealth;
			}
			
			//If we are in a menu there is no need to process input commands.
			if (inMenu)
				return;
				
				
			if (PBE.isKeyDown(InputKey.CONTROL))
			{
				//If the user pushes CTRL + M bring up the menu
				if (PBE.isKeyDown(InputKey.M))
				{
					PBE.screenManager.push("pawnMenu");
					PBE.soundManager.play("../assets/media/DrawWeapon.mp3", "guiSounds");
					inMenu = true;
				}
			}
			
			//Get the velocity position and scale of the player
			var velocity:Point = owner.getProperty(VelocityReference);
			var position:Point = owner.getProperty(PositionReference);
			var scale:Point = owner.getProperty(RenderReference);
			
			//Lets do our AI stuff
			if ( position )
			{
				playerPosition = position;
			}
			
			//Lets do our weapon delay
			if (weaponDelay > 0)
			{
				weaponDelay -= tickRate;
				weaponDelay = weaponDelay < 0 ? 0 : weaponDelay;
				if ( weaponDelay == 0 )
				{
					inAttack = false;
					animation = "idle";
					owner.eventDispatcher.dispatchEvent(new Event(AnimationEventName));
				}
				else
					inAttack = true;
			}
			
			//If the player is attacking pause him
			if (inAttack)
				return;
			
			if (PBE.isKeyDown(InputKey.RIGHT) || PBE.isKeyDown(InputKey.D))
			{
				if (!facing || idle) 
				{
					facing = true;
					idle = false;
					animation = "move";
					owner.eventDispatcher.dispatchEvent(new Event(AnimationEventName));
				}
				
				velocity.x = Speed;
			}
			else if (PBE.isKeyDown(InputKey.LEFT) || PBE.isKeyDown(InputKey.A))
			{							
				if (facing || idle) 
				{
					facing = false;
					idle = false;
					animation = "move";
					owner.eventDispatcher.dispatchEvent(new Event(AnimationEventName));
				}
				
				velocity.x = -Speed;

			}
			else if (!idle)
			{
				idle = true;
				velocity.x = 0;
				animation = "idle";
				owner.eventDispatcher.dispatchEvent(new Event(AnimationEventName));
			}	
			
			if (PBE.isKeyDown(InputKey.UP) || PBE.isKeyDown(InputKey.W))
			{
				
				if( onGround > 0)
				{
					velocity.y = -JumpSpeed;
					onGround = 0;
					isJumping = true;
					animation = "jump";
					owner.eventDispatcher.dispatchEvent(new Event(AnimationEventName));
				}
			}
			
			if (PBE.isKeyDown(InputKey.DOWN) || PBE.isKeyDown(InputKey.S))
			{
			}
			
			if (PBE.isKeyDown(InputKey.SPACE) && weaponDelay == 0)
			{
				PBE.soundManager.play("../assets/media/BladeSwing3.mp3", "inGameSounds");
				
				if ( PBE.isKeyDown(InputKey.DOWN) || PBE.isKeyDown(InputKey.S) && weaponDelay == 0 )
				{
					weaponDelay = WeaponDelay * 2;
					PBE.processManager.schedule(750, this, pawnAttack, position.add(WeaponOffset), facing, 100 );					
					animation = "attackTwo";
					owner.eventDispatcher.dispatchEvent(new Event(AnimationEventName));
				}
				else
				{
					weaponDelay = WeaponDelay;
					PBE.processManager.schedule(500, this, pawnAttack, position.add(WeaponOffset), facing, 50 );	
					animation = "attack";
					owner.eventDispatcher.dispatchEvent(new Event(AnimationEventName));
				}
			}
			
			//Depending on which way the pawn is facing will determine how the animations play for that character
			if ( facing )
			{
				scale = new Point( 0.5, 0.5 ); //Point( 1, 1 );
			}
			else
			{
				scale = new Point( -0.5, 0.5 ); //Point( -1, 1 );
			}
			owner.setProperty(VelocityReference, velocity);
			owner.setProperty(RenderReference, scale);
			return;
		}
		
		//Lets spawn the attack sword slash for the pawn and send it on its way
		private function pawnAttack( position:Point, facing:Boolean, damage:int) : void
		{
			var ThisNew2:IEntity = PBE.makeEntity("weaponFireball", 
						{
							"@Spatial.position": position,
							"@Spatial.linearVelocity": facing?new Point(800, -10):new Point(-800, -10),
							"@fireyBullet.Damage": damage,
							"@Render.scale":facing?new Point(-1, 1):new Point(1, 1)
						});
						
			var suicide2:ThinkingComponent = ThisNew2.lookupComponentByName("Suicide") as ThinkingComponent;
				suicide2.think(ThisNew2.destroy, 250);
			return;
		}

		protected override function onAdd():void
		{
			super.onAdd();
			owner.eventDispatcher.addEventListener(HealthComponent.DIED, onDied);
			owner.eventDispatcher.addEventListener(CollisionEvent.COLLISION_EVENT, OnCollision);
		}

		protected override function onRemove():void
		{
			
			owner.eventDispatcher.removeEventListener(HealthComponent.DIED, onDied);
			owner.eventDispatcher.removeEventListener(CollisionEvent.COLLISION_EVENT, OnCollision);
			super.onRemove();
		}

		protected function onDied(event:Event):void
		{
			animation = "death";
			owner.eventDispatcher.dispatchEvent(new Event(AnimationEventName));
			PBE.soundManager.play( "../assets/media/Death.mp3", "inGameSounds" );
			playerIsDead = true;
			
			//Make it over
			PBE.processManager.schedule(3000, Main, PBE.screenManager.push, "pawnGameOver" );
			return;
		}

		private function OnCollision(event:CollisionEvent):void
		{
			if ( PBE.objectTypeManager.doesTypeOverlap(event.collider.collisionType, "Level") )
			{
				if ( !playerChangeLevel )
				{
					//The player has touched the level time to load the next one
					playerChangeLevel = true;
					PBE.processManager.schedule(500, this, Main.nextLevel);
					return;
				}
			}
			
			if ( PBE.objectTypeManager.doesTypeOverlap(event.collidee.collisionType, "Player") )
			{
				if ( isJumping )
					isJumping = false;
			}
			
			if (PBE.objectTypeManager.doesTypeOverlap(event.collidee.collisionType, "Player") && PBE.objectTypeManager.doesTypeOverlap(event.collider.collisionType, "Platform"))
			{
				onGround = 1;
			}
			else if (PBE.objectTypeManager.doesTypeOverlap(event.collidee.collisionType, "Enemy"))
			{
				//player enemy collision does nothing
				onGround = 1;
			}
			return;
		}			
	}
}